// bkrmold.cpp - main
//
// Old Mail Arrangement Plug-in for Becky! 2
// Copyright (C) 2000 by Tietew
//
#include "pch.h"
#include "bkrmold.h"
#include "utys.h"
#include "remove.h"
#include "schedule.h"
#include <crtdbg.h>

#pragma comment(lib, "comdlg32.lib")
#pragma comment(lib, "version.lib")
#pragma comment(lib, "shell32.lib")

// Global Variables
CBeckyAPI bka;					// Becky API
HINSTANCE g_hInstance = NULL;	// hInstance of DLL
char g_szIni[MAX_PATH+2];		// Ini file to save your plugin settings.
UINT uID_Remove, uID_Schedule;	// Menu IDs
HIMAGELIST g_hilFolders;		// Folders ImageList
LONG g_nProcessing;				// Re-enter flag
COLORREF g_crUser[16];			// ChooseColor custom colors
HANDLE g_hHeap;

// Not to include CRT codes
#ifndef _DEBUG
#define DllMain _DllMainCRTStartup
#endif

#define BKID_NEWFOLDER	32804			// Folder > New
#define BKID_COMBO		32940			// Folder Mainteinance Combo
#define BKID_REPAIR		32798			// Repair
#define BKID_SORT		32776			// Arrangement by Sent Date

#define BKIDB_FOLDERS	130				// Folder Bitmap (Resource ID)

/////////////////////////////////////////////////////////////////////////////
// DLL entry point
extern "C" BOOL APIENTRY DllMain(HINSTANCE hModule, DWORD dwReason, LPVOID)
{
	if(dwReason == DLL_PROCESS_ATTACH)
	{
		// CRT Memory Leak Detection
		_CrtSetDbgFlag(_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG) | _CRTDBG_LEAK_CHECK_DF);
		
		// Initialize module and APIs
		if(!bka.InitAPI())
			return FALSE;
		DisableThreadLibraryCalls(hModule);
		g_hInstance = hModule;
		
		// Heap
		g_hHeap = HeapCreate(0, 4096, 0);
		if(!g_hHeap)
		{
			MessageBox(NULL, "OUT OF MEMORY!", "bkrmold", MB_ICONHAND | MB_OK | MB_SYSTEMMODAL);
			TerminateProcess(GetCurrentProcess(), -1);
		}
		
		// Load resource DLL
		char module[MAX_PATH + 5];
		GetModuleFileName(hModule, module, MAX_PATH);
		
		// Get path to store ini file
		char path[MAX_PATH];
		lstrcpy(path, bka.GetDataFolder());
		lstrcat(path, "PlugIns");
		
		WIN32_FIND_DATA fd;
		HANDLE hFind = FindFirstFile(path, &fd);
		if(hFind != INVALID_HANDLE_VALUE)
		{
			if(!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				MyMessageBox(NULL, IDS_FAIL_INI, MB_OK | MB_ICONHAND | MB_SYSTEMMODAL);
				return FALSE;
			}
			FindClose(hFind);
		}
		else
			CreateDirectory(path, NULL);
		int idx = lstrlen(path);
		path[idx] = '\\';
		PathRenameExtension(module, ".ini");
		lstrcpy(path + idx + 1, PathFindFileName(module));
		lstrcpy(g_szIni, path);
		
		// Set update URL
		char url[MAX_PATH];
		LoadString(g_hInstance, IDS_UPDATE_URL, url, MAX_PATH);
		WritePrivateProfileString(INI_SEC_BKUPDATE, INI_KEY_URL, url, g_szIni);
		
		// Load custom color
		for(int i = 0; i < countof(g_crUser); ++i)
		{
			char key[32], value[16];
			wsprintf(key, "Color%d", i);
			LPSTR str = value;
			GetPrivateProfileString(INI_SECTION, key, "", value, countof(value), g_szIni);
			g_crUser[i] = hex32(str) & 0x00FFFFFF;
		}
		
		// Initialize menu commands
		char szHintRemove[64], szHintSchedule[64];
		LoadString(g_hInstance, IDS_HINT_REMOVE, szHintRemove, sizeof szHintRemove);
		LoadString(g_hInstance, IDS_HINT_SCHEDULE, szHintSchedule, sizeof szHintSchedule);
		uID_Remove = bka.RegisterCommand(szHintRemove, BKC_MENU_TREEVIEW/*nType*/, OnRemoveMails);
		uID_Schedule = bka.RegisterCommand(szHintSchedule, BKC_MENU_TREEVIEW/*nType*/, OnSchedule);
		bka.RegisterUICallback(uID_Remove, OnUpdateRemoveMails);
		bka.RegisterUICallback(uID_Schedule, OnUpdateSchedule);
		
		// Initilalize DateTimePicker control
		INITCOMMONCONTROLSEX iccx;
		iccx.dwSize = sizeof iccx;
		iccx.dwICC = ICC_DATE_CLASSES;
		InitCommonControlsEx(&iccx);
		
		// Get folder icons and create ImageList
		g_hilFolders = ImageList_LoadImage(bka.GetB2Instance(), MAKEINTRESOURCE(BKIDB_FOLDERS),
			16, 1, CLR_DEFAULT/*RGB(255,0,255)*/, IMAGE_BITMAP, 0);
		if(!g_hilFolders)
			return FALSE;
	}
	else if(dwReason == DLL_PROCESS_DETACH)
	{
		// Save custom colors
		for(int i = 0; i < countof(g_crUser); ++i)
		{
			char key[32];
			wsprintf(key, "Color%d", i);
			WritePrivateProfileString(INI_SECTION, key, hex32(g_crUser[i]), g_szIni);
		}
		ImageList_Destroy(g_hilFolders);
#ifndef _DEBUG
		HeapDestroy(g_hHeap);
#endif
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////
// Called when the program is started and the main window is created.
int WINAPI BKC_OnStart()
{
	// Always return 0.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when the main window is closing.

// Process schedule and call recursively for each sub folders
static void ProcessFolderSchedule(String path, String folder)
{
	WIN32_FIND_DATA fd;
	HANDLE hFind = FindFirstFile(String(path + folder + '*'), &fd);
	if(hFind == INVALID_HANDLE_VALUE)
		return;
	do
		if((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			&& lstrcmp(fd.cFileName, ".") != 0 && lstrcmp(fd.cFileName, "..") != 0)
		{
			String thefolder = folder + fd.cFileName + '\\';
			DoSchedule(thefolder, IDC_ONEND);
			ProcessFolderSchedule(path, thefolder);
		}
	while(FindNextFile(hFind, &fd));
}

int WINAPI BKC_OnExit()
{
	// Re-enter check
	if(InterlockedIncrement(&g_nProcessing) != 1)
		return 0;
	
	if(GetIniExitSchedule())
	{
		BOOL bLockout = GetIniLockout();
		if(bLockout)
		{
			SCHEDULEINFO si;
			si.bLockout = true;
			si.nLockoutNum = GetIniLockoutNum();
			si.nLockoutUnit = GetIniLockoutUnit();
			if(IsLockout(si, g_szIni))
				return 0;
		}
		CBeckyWindow wnd;
		
		char msg[64];
		LoadString(g_hInstance, IDS_PROCESS_SCHEDULE, msg, countof(msg));
		bka.SetMessageText(NULL, msg);
		
		// Disabled main window and set controls not to redraw
		EnableWindow(wnd.hwndMain, FALSE);
		SendMessage(wnd.hwndTree, WM_SETREDRAW, FALSE, 0);
		SendMessage(wnd.hwndList, WM_SETREDRAW, FALSE, 0);
		SendMessage(wnd.hwndView, WM_SETREDRAW, FALSE, 0);
		SendMessage(wnd.hwndStatus, WM_SETREDRAW, FALSE, 0);
		
		// Process for each mailbox
		String path = bka.GetDataFolder();
		WIN32_FIND_DATA fd;
		HANDLE hFind = FindFirstFile(String(path + '*'), &fd);
		if(hFind != INVALID_HANDLE_VALUE)
		{
			do
				if((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
					&& lstrcmpi(PathFindExtension(fd.cFileName), ".mb") == 0)
				{
					ProcessFolderSchedule(path, String(fd.cFileName) + '\\');
				}
			while(FindNextFile(hFind, &fd));
			FindClose(hFind);
		}
		if(bLockout)
			WriteCurTime(g_szIni);
	}
	// Return -1 if you don't want to quit.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when menu is intialized.
void InitMenu(HMENU hMenu)
{
	char szMenuRemove[64], szMenuSchedule[64];
	LoadString(g_hInstance, IDS_MENU_REMOVE, szMenuRemove, sizeof szMenuRemove);
	LoadString(g_hInstance, IDS_MENU_SCHEDULE, szMenuSchedule, sizeof szMenuSchedule);
	
	int i, max = GetMenuItemCount(hMenu);
	for(i = 0; i < max; ++i)
		if(GetMenuItemID(hMenu, i) == BKID_REPAIR)
			break;
	InsertMenu(hMenu, i + 1, MF_BYPOSITION | MF_SEPARATOR, 0, NULL);
	InsertMenu(hMenu, i + 2, MF_BYPOSITION | MF_STRING, uID_Remove, szMenuRemove);
	InsertMenu(hMenu, i + 3, MF_BYPOSITION | MF_STRING, uID_Schedule, szMenuSchedule);
}

int WINAPI BKC_OnMenuInit(HWND hWnd, HMENU hMenu, int nType)
{
	if(nType == BKC_MENU_TREEVIEW)
		InitMenu(hMenu);
	else if(nType == BKC_MENU_MAIN)
		InitMenu(GetSubMenu(GetSubMenu(hMenu, 0), 0));
	// Always return 0.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when a folder is opened.
int WINAPI BKC_OnOpenFolder(LPCTSTR lpFolderID)
{
	// Re-enter check
	if(InterlockedIncrement(&g_nProcessing) == 1)
		DoSchedule(lpFolderID, IDC_ONSELECT);
	InterlockedDecrement(&g_nProcessing);
	// Always return 0.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when a mail is selected.
int WINAPI BKC_OnOpenMail(LPCTSTR lpMailID)
{
	// Always return 0.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called every minute.
int WINAPI BKC_OnEveryMinute()
{
	// Always return 0.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when a compose windows is opened.
int WINAPI BKC_OnOpenCompose(HWND hWnd, int nMode/* See COMPOSE_MODE_* in BeckyApi.h */)
{
	// Always return 0.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when the composing message is saved.
int WINAPI BKC_OnOutgoing(HWND hWnd, int nMode/* 0:SaveToOutbox, 1:SaveToDraft, 2:SaveToReminder*/) 
{
	// Return -1 if you do not want to send it yet.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when a key is pressed.
int WINAPI BKC_OnKeyDispatch(HWND hWnd, int nKey/* virtual key code */, int nShift/* Shift state. 0x40=Shift, 0x20=Ctrl, 0x60=Shift+Ctrl, 0xfe=Alt*/)
{
	WORD hotkey;
	
	if(nShift == 0x40)
		hotkey = MAKEWORD(nKey, HOTKEYF_SHIFT);
	else if(nShift == 0x20)
		hotkey = MAKEWORD(nKey, HOTKEYF_CONTROL);
	else if(nShift == 0x60)
		hotkey = MAKEWORD(nKey, HOTKEYF_SHIFT | HOTKEYF_CONTROL);
	
	CBeckyWindow wnd;
	if(hotkey == GetIniHotArrange())
	{
		if(OnUpdateRemoveMails(wnd.hwndMain, uID_Remove) != BKMENU_CMDUI_DISABLED)
			OnRemoveMails(wnd.hwndMain, uID_Remove);
	}
	else if(hotkey == GetIniHotSchedule())
	{
		if(OnUpdateSchedule(wnd.hwndMain, uID_Schedule) != BKMENU_CMDUI_DISABLED)
			OnSchedule(wnd.hwndMain, uID_Schedule);
	}
	else
		return 0;
	
	// Return TRUE if you want to suppress subsequent command associated to this key.
	return TRUE;
}


////////////////////////////////////////////////////////////////////////
// Called when a message is retrieved and saved to a folder
int WINAPI BKC_OnRetrieve(LPCTSTR lpMessage/* Message source*/, LPCTSTR lpMailID/* Mail ID*/)
{
	// Always return 0.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when a message is spooled
int WINAPI BKC_OnSend(LPCTSTR lpMessage/* Message source */)
{
	// Return BKC_ONSEND_PROCESSED, if you have processed this message
	// and don't need Becky! to send it.
	// Becky! will move this message to Sent box when the sending
	// operation is done.
	// CAUTION: You are responsible for the destination of this
	// message if you return BKC_ONSEND_PROCESSED.
	
	// Return BKC_ONSEND_ERROR, if you want to cancel the sending operation.
	// You are responsible for displaying an error message.
	
	// Return 0 to proceed the sending operation.
	return 0;
}

////////////////////////////////////////////////////////////////////////
// Called when all messages are retrieved
int WINAPI BKC_OnFinishRetrieve(int nNumber/* Number of messages*/)
{
	// Always return 0.
	return 0;
}


////////////////////////////////////////////////////////////////////////
// Called when plug-in setup is needed.
int WINAPI BKC_OnPlugInSetup(HWND hWnd)
{
	SettingDlg(hWnd);
	// Return nonzero if you have processed.
	return 1;
}


////////////////////////////////////////////////////////////////////////
// Called when plug-in information is being retrieved.
typedef struct tagBKPLUGININFO
{
	char szPlugInName[80]; // Name of the plug-in
	char szVendor[80]; // Name of the vendor
	char szVersion[80]; // Version string
	char szDescription[256]; // Short description about this plugin
} BKPLUGININFO, *LPBKPLUGININFO;

int WINAPI BKC_OnPlugInInfo(LPBKPLUGININFO lpPlugInInfo)
{
	LoadString(g_hInstance, IDS_PLUGINNAME, lpPlugInInfo->szPlugInName, 80);
	LoadString(g_hInstance, IDS_VENDOR, lpPlugInInfo->szVendor, 80);
	LoadString(g_hInstance, IDS_DESCRIPTION, lpPlugInInfo->szDescription, 256);
	GetVersionString(lpPlugInInfo->szVersion);
	// Always return 0.
	return 0;
}

////////////////////////////////////////////////////////////////////////
