// filter.cpp - Advanced Filter
//
// Old Mail Arrangement Plug-in for Becky! 2
// Copyright (C) 2000 by Tietew
//
#include "pch.h"
#include "bkrmold.h"
#include "schedule.h"
#include "filter.h"
#include "utys.h"

// ColorLabel table
static COLORREF colors[] = {
	RGB(  0,   0,   0),			// Black
	RGB(128,   0,   0),			// Maroon
	RGB(  0, 128,   0),			// Green
	RGB(128, 128,   0),			// Olive
	RGB(  0,   0, 128),			// Navy
	RGB(128,   0, 128),			// Purple
	RGB(  0, 128, 128),			// Teal
	RGB(128, 128, 128),			// Gray
	RGB(192, 192, 192),			// Silver
	RGB(255,   0,   0),			// Red
	RGB(  0, 255,   0),			// Lime
	RGB(255, 255,   0),			// Yellow
	RGB(  0,   0, 255),			// Blue
	RGB(255,   0, 255),			// Fuchsia
	RGB(  0, 255, 255),			// Aqua
	RGB(255, 255, 255),			// White
	RGB(  0,   0,   0),			// (Other)
};

String GetString(LPCSTR& str)
{
	String r;
	LPCSTR last = str;
	while(*last && *last != '\1' && *last != '|')
	{
		if(*last == '\\')
			if(last[1] == '|' || last[1] == '\\')
				++last;
			else
				break;
		++last;
	}
	LPSTR pch = r.Alloc(last - str);
	while(str != last)
	{
		if(*str == '\\')
			if(str[1] == '|' || str[1] == '\\')
				++str;
		*pch++ = *str++;
	}
	++str;
	*pch = '\0';
	return r.Fix();
}

// Get a text filter item
bool GetFilterText(FILTERTEXT& ft, LPCSTR& str)
{
	switch(*str)
	{
		case '-':
			ft.bNot = ft.bRegExp = false;
			break;
		case 'R':
			ft.bNot = false;
			ft.bRegExp = true;
			break;
		case 'r':
			ft.bNot = ft.bRegExp = true;
			break;
		case 'X':
			ft.bNot = true;
			ft.bRegExp = false;
			break;
		case '\1':
		case '|':
			++str;
		case '\0':
			return false;
	}
	++str;
	ft.text = GetString(str);
	return ft.text != String::Nil;
}

inline int GetFilterItemChar(LPCSTR& str, const char *pch, UINT len, int def = 0, bool bCharOnly = false)
{
	if(!bCharOnly && (unsigned char)(*str - '0') < len)
		return *str++ - '0';
	char *found = (char*)memchr(pch, *str, len);
	if(!found)
		return def;
	++str;
	return found - pch;
}

inline void skip(LPCSTR& str)
{
	while(*str && *str != '\1' && *str != '|')
		++str;
	++str;
}

// Get a filter item
void GetFilterItem(FILTERINFO& fi, String path, int i)
{
	char key[32];
	wsprintf(key, "Filter%d", i);
	
	DWORD dwSize = 256, dwRead;
	LPSTR value = new char[256];
	while(1)
	{
		dwRead = GetPrivateProfileString(INI_SECTION, key, "", value, dwSize, path);
		if(dwRead < dwSize - 1)
			break;
		delete[] value;
		value = new char[dwSize *= 2];
	}
	if(*value)
	{
		LPSTR str = value;
		fi.v.push_back(FILTERITEM(GetString(str)));
		FILTERITEM& fim = fi.v.back();
		if(GetFilterText(fim.from, str))
			fim.dwFlag |= FILTER_FROM;
		if(GetFilterText(fim.to, str))
			fim.dwFlag |= FILTER_TO;
		if(GetFilterText(fim.subject, str))
			fim.dwFlag |= FILTER_SUBJECT;
		if(*str == '\1' || *str == '|')
			++str;
		else if(*str)
		{
			fim.dwFlag |= FILTER_PRIORITY;
			fim.nPriority = GetFilterItemChar(str, "HhnlL", 5);
			fim.nPriorityUnit = GetFilterItemChar(str, "=<>", 3);
			skip(str);
		}
		if(*str == '\1' || *str == '|')
			++str;
		else if(*str)
		{
			fim.dwFlag |= FILTER_COLORLABEL;
			fim.color = (COLORREF)(hex32(str) & 0x00FFFFFF);
			skip(str);
		}
		if(*str == '\1' || *str == '|')
			++str;
		else if(*str)
		{
			fim.dwFlag |= FILTER_STATUS;
			fim.nStatus = GetFilterItemChar(str, "RU", 2);
			skip(str);
		}
		if(*str == '\1' || *str == '|')
			++str;
		else if(*str)
		{
			fim.dwFlag |= FILTER_SIZE;
			fim.nSizeUnit = GetFilterItemChar(str, "<>", 2, 0, true);
			fim.nSize = (int)hex32(str);
		}
	}
	delete[] value;
}

void AppendString(String& value, String str)
{
	int len = value.length();
	LPSTR pch = value.Alloc(len + str.length() * 2) + len;
	LPCSTR src = str;
	while(*src)
	{
		if(*src == '|' || *src == '\\')
			*pch++ = '\\';
		*pch++ = *src++;
	}
	*pch = '\0';
	value.Fix();
}

// Set a text filter item
void SetFilterText(String& value, const FILTERTEXT& ft, BOOL b)
{
	if(b && ft.text != String::Nil)
	{
		value += ft.bNot ? (ft.bRegExp ? 'r' : 'X') : (ft.bRegExp ? 'R' : '-');
		AppendString(value, ft.text);
	}
	value += '|';
}

// Set a filter item
void SetFilterItem(const FILTERITEM& item, int i, String path)
{
	String value;
	AppendString(value, item.name);
	value += '|';
	SetFilterText(value, item.from, item.dwFlag & FILTER_FROM);
	SetFilterText(value, item.to, item.dwFlag & FILTER_TO);
	SetFilterText(value, item.subject, item.dwFlag & FILTER_SUBJECT);
	if(item.dwFlag & FILTER_PRIORITY)
	{
		value += "HhnlL"[item.nPriority];
		value += "=<>"[item.nPriorityUnit];
	}
	value += '|';
	if(item.dwFlag & FILTER_COLORLABEL)
		value += hex32(item.color);
	value += '|';
	if(item.dwFlag & FILTER_STATUS)
		value += "RU"[item.nStatus];
	value += '|';
	if(item.dwFlag & FILTER_SIZE)
	{
		value += "<>"[item.nSizeUnit];
		value += hex32(item.nSize);
	}
	value += '|';
	char key[32];
	wsprintf(key, "Filter%d", i);
	WritePrivateProfileString(INI_SECTION, key, value, path);
}

// initialize name selector
void InitFilterCombo(HWND hwnd, UINT nID, UINT nIDFirst, UINT nIFLast)
{
	char str[256];
	for(UINT i = nIDFirst; i <= nIFLast; ++i)
	{
		LoadString(g_hInstance, i, str, countof(str));
		SendDlgItemMessage(hwnd, nID, CB_ADDSTRING, 0, (LPARAM)str);
	}
}

// Select a color
void SelectColorCombo(HWND hwnd, UINT nID, COLORREF cr)
{
	int i = 0;
	if(cr != 0)
	{
		for(i = 1; i < countof(colors) - 1; ++i)
			if(colors[i] == cr)
				break;
		if(i == countof(colors) - 1)
			SendDlgItemMessage(hwnd, nID, CB_SETITEMDATA, i, (LPARAM)cr);
	}
	SendDlgItemMessage(hwnd, nID, CB_SETCURSEL, i, 0);
}

void FixFilterTextData(FILTERTEXT& ft, HWND hwnd, UINT nIDText, UINT nIDNot, UINT nIDRegExp)
{
	int len = GetWindowTextLength(GetDlgItem(hwnd, nIDText));
	LPSTR str = ft.text.Alloc(len);
	len = GetDlgItemText(hwnd, nIDText, str, len + 1);
	ft.text.Fix(len);
	ft.bNot = IsDlgButtonChecked2(hwnd, nIDNot);
	ft.bRegExp = IsDlgButtonChecked2(hwnd, nIDRegExp);
}

void FixData(HWND hwnd)
{
	FILTERDLGINFO& fdi = *(FILTERDLGINFO*)GetWindowLong(hwnd, DWL_USER);
	FILTERITEM& fi = *(FILTERITEM*)SendDlgItemMessage(hwnd, IDC_NAME, CB_GETITEMDATA, fdi.nCurSel, 0);
	fi.dwFlag = 0;
	if(IsDlgButtonChecked2(hwnd, IDC_FROM))
	{
		fi.dwFlag |= FILTER_FROM;
		FixFilterTextData(fi.from, hwnd, IDC_FROMTEXT, IDC_FROMNOT, IDC_FROMRE);
	}
	if(IsDlgButtonChecked2(hwnd, IDC_TO))
	{
		fi.dwFlag |= FILTER_TO;
		FixFilterTextData(fi.to, hwnd, IDC_TOTEXT, IDC_TONOT, IDC_TORE);
	}
	if(IsDlgButtonChecked2(hwnd, IDC_SUBJECT))
	{
		fi.dwFlag |= FILTER_SUBJECT;
		FixFilterTextData(fi.subject, hwnd, IDC_SUBJECTTEXT, IDC_SUBJECTNOT, IDC_SUBJECTRE);
	}
	if(IsDlgButtonChecked2(hwnd, IDC_PRIORITY))
	{
		fi.dwFlag |= FILTER_PRIORITY;
		fi.nPriority = SendDlgItemMessage(hwnd, IDC_PRIORITYCOMBO, CB_GETCURSEL, 0, 0);
		fi.nPriorityUnit = SendDlgItemMessage(hwnd, IDC_PRIORITYUNIT, CB_GETCURSEL, 0, 0);
	}
	if(IsDlgButtonChecked2(hwnd, IDC_COLORLABEL))
	{
		fi.dwFlag |= FILTER_COLORLABEL;
		int idx = SendDlgItemMessage(hwnd, IDC_COLORCOMBO, CB_GETCURSEL, 0, 0);
		fi.color = (COLORREF)SendDlgItemMessage(hwnd, IDC_COLORCOMBO, CB_GETITEMDATA, idx, 0);
	}
	if(IsDlgButtonChecked2(hwnd, IDC_STATUS))
	{
		fi.dwFlag |= FILTER_STATUS;
		fi.nStatus = SendDlgItemMessage(hwnd, IDC_STATUSCOMBO, CB_GETCURSEL, 0, 0);
	}
	if(IsDlgButtonChecked2(hwnd, IDC_FSIZE))
	{
		fi.dwFlag |= FILTER_SIZE;
		fi.nSize = GetDlgItemInt(hwnd, IDC_SIZETEXT, NULL, FALSE);
		fi.nSizeUnit = SendDlgItemMessage(hwnd, IDC_SIZEUNIT, CB_GETCURSEL, 0, 0);
	}
}

BOOL CALLBACK FilterProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			FILTERDLGINFO& fdi = *(FILTERDLGINFO*)lp;
			SetWindowLong(hwnd, DWL_USER, lp);
			if(fdi.fi.v.size() == 0)
				fdi.fi.v.push_back(FILTERITEM("Filter1"));
			SendDlgItemMessage(hwnd, IDC_SIZESPIN, UDM_SETRANGE32, 0, INT_MAX);
			
			// Initialize name selector
			int idx;
			HWND hCombo = GetDlgItem(hwnd, IDC_NAME);
			for(FILTERINFO::TList::iterator it = fdi.fi.v.begin(); it != fdi.fi.v.end(); ++it)
			{
				idx = SendMessage(hCombo, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)it->name);
				SendMessage(hCombo, CB_SETITEMDATA, idx, (LPARAM)&*it);
			}
			SendMessage(hCombo, CB_SETCURSEL, 0, 0);
			fdi.nCurSel = 0;
			
			InitFilterCombo(hwnd, IDC_PRIORITYCOMBO, IDS_PRIORITY_HIGH, IDS_PRIORITY_LOW);
			InitFilterCombo(hwnd, IDC_PRIORITYUNIT, IDS_PRIORITY_UNIT_EQ, IDS_PRIORITY_UNIT_LEQ);
			InitFilterCombo(hwnd, IDC_STATUSCOMBO, IDS_STATUS_READ, IDS_STATUS_UNREAD);
			InitFilterCombo(hwnd, IDC_SIZEUNIT, IDS_SIZE_UNIT_OVER, IDS_SIZE_UNIT_LESS);
			
			// Initialize color selector
			char str[256];
			colors[0] = GetSysColor(COLOR_WINDOWTEXT);
			for(int i = 0; i < countof(colors); ++i)
			{
				LoadString(g_hInstance, IDS_COLOR_BLACK + i, str, countof(str));
				idx = SendDlgItemMessage(hwnd, IDC_COLORCOMBO, CB_ADDSTRING, 0, (LPARAM)str);
				SendDlgItemMessage(hwnd, IDC_COLORCOMBO, CB_SETITEMDATA, idx, colors[i]);
			}
			
			PostMessage(hwnd, WM_COMMAND, MAKELONG(IDC_NAME, CBN_SELCHANGE), (LPARAM)hCombo);
			return FALSE;
		}
		case WM_COMMAND:
			switch(LOWORD(wp))
			{
				case IDOK:
				{
					FixData(hwnd);
					// Remove all NO-MEANS filters
					FILTERDLGINFO& fdi = *(FILTERDLGINFO*)GetWindowLong(hwnd, DWL_USER);
					FILTERINFO::TList::iterator it, here;
					for(it = fdi.fi.v.begin(); it != fdi.fi.v.end();)
					{
						here = it++;
						if((here->dwFlag & FILTER_FROM) && here->from.text == String::Nil)
							here->dwFlag &= ~FILTER_FROM;
						if((here->dwFlag & FILTER_TO) && here->to.text == String::Nil)
							here->dwFlag &= ~FILTER_TO;
						if((here->dwFlag & FILTER_SUBJECT) && here->subject.text == String::Nil)
							here->dwFlag &= ~FILTER_SUBJECT;
						if((here->dwFlag & FILTER_SIZE) && here->nSize == 0)
							here->dwFlag &= ~FILTER_SIZE;
						if(here->dwFlag == 0)
							fdi.fi.v.erase(here);
					}
				}
				// fall through
				case IDCANCEL:
					EndDialog(hwnd, LOWORD(wp));
					return TRUE;
				
				case IDC_NAME:
					if(HIWORD(wp) == CBN_SELCHANGE)
					{
						int idx = SendMessage((HWND)lp, CB_GETCURSEL, 0, 0);
						FILTERITEM& fi = *(FILTERITEM*)SendMessage((HWND)lp, CB_GETITEMDATA, idx, 0);
						FILTERDLGINFO& fdi = *(FILTERDLGINFO*)GetWindowLong(hwnd, DWL_USER);
						fdi.nCurSel = idx;
						CheckDlgButton2(hwnd, IDC_FROM, fi.dwFlag & FILTER_FROM);
						SetDlgItemText(hwnd, IDC_FROMTEXT, fi.from.text);
						CheckDlgButton2(hwnd, IDC_FROMNOT, fi.from.bNot);
						CheckDlgButton2(hwnd, IDC_FROMRE, fi.from.bRegExp);
						CheckDlgButton2(hwnd, IDC_TO, fi.dwFlag & FILTER_TO);
						SetDlgItemText(hwnd, IDC_TOTEXT, fi.to.text);
						CheckDlgButton2(hwnd, IDC_TONOT, fi.to.bNot);
						CheckDlgButton2(hwnd, IDC_TORE, fi.to.bRegExp);
						CheckDlgButton2(hwnd, IDC_SUBJECT, fi.dwFlag & FILTER_SUBJECT);
						SetDlgItemText(hwnd, IDC_SUBJECTTEXT, fi.subject.text);
						CheckDlgButton2(hwnd, IDC_SUBJECTNOT, fi.subject.bNot);
						CheckDlgButton2(hwnd, IDC_SUBJECTRE, fi.subject.bRegExp);
						CheckDlgButton2(hwnd, IDC_PRIORITY, fi.dwFlag & FILTER_PRIORITY);
						SendDlgItemMessage(hwnd, IDC_PRIORITYCOMBO, CB_SETCURSEL, fi.nPriority, 0);
						SendDlgItemMessage(hwnd, IDC_PRIORITYUNIT, CB_SETCURSEL, fi.nPriorityUnit, 0);
						CheckDlgButton2(hwnd, IDC_COLORLABEL, fi.dwFlag & FILTER_COLORLABEL);
						SelectColorCombo(hwnd, IDC_COLORCOMBO, fi.color);
						CheckDlgButton2(hwnd, IDC_STATUS, fi.dwFlag & FILTER_STATUS);
						SendDlgItemMessage(hwnd, IDC_STATUSCOMBO, CB_SETCURSEL, fi.nStatus, 0);
						CheckDlgButton2(hwnd, IDC_FSIZE, fi.dwFlag & FILTER_SIZE);
						SetDlgItemInt(hwnd, IDC_SIZETEXT, fi.nSize, FALSE);
						SendDlgItemMessage(hwnd, IDC_SIZEUNIT, CB_SETCURSEL, fi.nSizeUnit, 0);
						PostMessage(hwnd, WM_COMMAND, IDC_FROM, 0);
						PostMessage(hwnd, WM_COMMAND, IDC_TO, 0);
						PostMessage(hwnd, WM_COMMAND, IDC_SUBJECT, 0);
						PostMessage(hwnd, WM_COMMAND, IDC_PRIORITY, 0);
						PostMessage(hwnd, WM_COMMAND, IDC_COLORLABEL, 0);
						PostMessage(hwnd, WM_COMMAND, IDC_STATUS, 0);
						PostMessage(hwnd, WM_COMMAND, IDC_FSIZE, 0);
						return TRUE;
					}
					else if(HIWORD(wp) == CBN_DROPDOWN || HIWORD(wp) == CBN_KILLFOCUS || HIWORD(wp) == CBN_SETFOCUS)
					{
						FixData(hwnd);
						
						HWND hCombo = (HWND)lp;
						FILTERDLGINFO& fdi = *(FILTERDLGINFO*)GetWindowLong(hwnd, DWL_USER);
						FILTERITEM *pfi = (FILTERITEM*)SendMessage(hCombo, CB_GETITEMDATA, fdi.nCurSel, 0);
						int len = GetWindowTextLength(hCombo);
						char *str = (char*)_alloca(len + 1);
						GetWindowText(hCombo, str, len + 1);
						pfi->name = str;
						SendMessage(hCombo, CB_DELETESTRING, fdi.nCurSel, 0);
						SendMessage(hCombo, CB_INSERTSTRING, fdi.nCurSel, (LPARAM)str);
						SendMessage(hCombo, CB_SETITEMDATA, fdi.nCurSel, (LPARAM)pfi);
						SendMessage(hCombo, CB_SETCURSEL, fdi.nCurSel, 0);
						
						return TRUE;
					}
					break;
				
				case IDC_ADD:
				{
					// Add a filter item
					FixData(hwnd);
					
					FILTERDLGINFO& fdi = *(FILTERDLGINFO*)GetWindowLong(hwnd, DWL_USER);
					HWND hCombo = GetDlgItem(hwnd, IDC_NAME);
					int idx = SendMessage(hCombo, CB_GETCOUNT, 0, 0);
					char str[64];
					wsprintf(str, "Filter%d", idx + 1);
					fdi.fi.v.push_back(FILTERITEM(str));
					idx = SendMessage(hCombo, CB_ADDSTRING, 0, (LPARAM)str);
					SendMessage(hCombo, CB_SETITEMDATA, idx, (LPARAM)&fdi.fi.v.back());
					SendMessage(hCombo, CB_SETCURSEL, idx, 0);
					SendMessage(hwnd, WM_COMMAND, MAKELONG(IDC_NAME, CBN_SELCHANGE), (LPARAM)hCombo);
					
					return TRUE;
				}
				case IDC_DELETE:
				{
					// Delete a current filter item.
					HWND hCombo = GetDlgItem(hwnd, IDC_NAME);
					FILTERDLGINFO& fdi = *(FILTERDLGINFO*)GetWindowLong(hwnd, DWL_USER);
					FILTERITEM *pfi = (FILTERITEM*)SendMessage(hCombo, CB_GETITEMDATA, fdi.nCurSel, 0);
					FILTERINFO::TList::iterator it = fdi.fi.v.begin();
					advance(it, fdi.nCurSel);
					fdi.fi.v.erase(it);
					SendDlgItemMessage(hwnd, IDC_NAME, CB_DELETESTRING, fdi.nCurSel, 0);
					if(fdi.fi.v.size() == 0)
					{
						// No filter item now: create dummy item.
						fdi.fi.v.push_back(FILTERITEM("Filter1"));
						int idx = SendMessage(hCombo, CB_ADDSTRING, 0, (LPARAM)"Filter1");
						SendMessage(hCombo, CB_SETITEMDATA, idx, (LPARAM)&fdi.fi.v.back());
						fdi.nCurSel = 0;
					}
					if(fdi.fi.v.size() >= fdi.nCurSel)
						fdi.nCurSel = fdi.fi.v.size() - 1;
					SendMessage(hCombo, CB_SETCURSEL, fdi.nCurSel, 0);
					SendMessage(hwnd, WM_COMMAND, MAKELONG(IDC_NAME, CBN_SELCHANGE), (LPARAM)hCombo);
					
					return TRUE;
				}
				case IDC_FROM:
				{
					bool bEnable = IsDlgButtonChecked2(hwnd, IDC_FROM);
					EnableWindow(GetDlgItem(hwnd, IDC_FROMTEXT), bEnable);
					EnableWindow(GetDlgItem(hwnd, IDC_FROMNOT), bEnable);
				//	EnableWindow(GetDlgItem(hwnd, IDC_FROMRE), bEnable);
					return TRUE;
				}
				case IDC_TO:
				{
					bool bEnable = IsDlgButtonChecked2(hwnd, IDC_TO);
					EnableWindow(GetDlgItem(hwnd, IDC_TOTEXT), bEnable);
					EnableWindow(GetDlgItem(hwnd, IDC_TONOT), bEnable);
				//	EnableWindow(GetDlgItem(hwnd, IDC_TORE), bEnable);
					return TRUE;
				}
				case IDC_SUBJECT:
				{
					bool bEnable = IsDlgButtonChecked2(hwnd, IDC_SUBJECT);
					EnableWindow(GetDlgItem(hwnd, IDC_SUBJECTTEXT), bEnable);
					EnableWindow(GetDlgItem(hwnd, IDC_SUBJECTNOT), bEnable);
				//	EnableWindow(GetDlgItem(hwnd, IDC_SUBJECTRE), bEnable);
					return TRUE;
				}
				case IDC_PRIORITY:
				{
					bool bEnable = IsDlgButtonChecked2(hwnd, IDC_PRIORITY);
					EnableWindow(GetDlgItem(hwnd, IDC_PRIORITYCOMBO), bEnable);
					EnableWindow(GetDlgItem(hwnd, IDC_PRIORITYUNIT), bEnable);
					return TRUE;
				}
				case IDC_COLORLABEL:
					EnableWindow(GetDlgItem(hwnd, IDC_COLORCOMBO),
						IsDlgButtonChecked2(hwnd, IDC_COLORLABEL));
					return TRUE;
				
				case IDC_STATUS:
					EnableWindow(GetDlgItem(hwnd, IDC_STATUSCOMBO),
						IsDlgButtonChecked2(hwnd, IDC_STATUS));
					return TRUE;
				
				case IDC_FSIZE:
				{
					bool bEnable = IsDlgButtonChecked2(hwnd, IDC_FSIZE);
					EnableWindow(GetDlgItem(hwnd, IDC_SIZETEXT), bEnable);
					EnableWindow(GetDlgItem(hwnd, IDC_STATIC_SIZE), bEnable);
					EnableWindow(GetDlgItem(hwnd, IDC_SIZEUNIT), bEnable);
					return TRUE;
				}
				case IDC_COLORCOMBO:
					if(HIWORD(wp) == CBN_SELCHANGE)
					{
						HWND hCombo = (HWND)lp;
						int n = SendMessage(hCombo, CB_GETCURSEL, 0, 0);
						if(n != countof(colors) - 1)
							return TRUE;
						
						// Chosen Others...
						COLORREF cr = (COLORREF)SendMessage(hCombo, CB_GETITEMDATA, n, 0);
						CHOOSECOLOR cc;
						cc.lStructSize = sizeof cc;
						cc.hwndOwner = hwnd;
						cc.rgbResult = cr;
						cc.lpCustColors = g_crUser;
						cc.Flags = CC_ANYCOLOR | CC_RGBINIT;
						if(ChooseColor(&cc))
							SendMessage(hCombo, CB_SETITEMDATA, n, (LPARAM)cc.rgbResult);
						InvalidateRect(hCombo, NULL, TRUE);
						return TRUE;
					}
					break;
			}
			break;
		
		case WM_MEASUREITEM:
		{
			MEASUREITEMSTRUCT& mis = *(MEASUREITEMSTRUCT*)lp;
			HFONT hFont = (HFONT)SendMessage(GetParent(hwnd), WM_GETFONT, 0, 0);
			HDC dc = GetDC(hwnd);
			HGDIOBJ hOldFont = SelectObject(dc, hFont);
			TEXTMETRIC tm;
			GetTextMetrics(dc, &tm);
			mis.itemHeight = tm.tmHeight + tm.tmInternalLeading + 2;
			SelectObject(dc, hOldFont);
			ReleaseDC(hwnd, dc);
			return TRUE;
		}
		case WM_DRAWITEM:
		{
			DRAWITEMSTRUCT& dis = *(DRAWITEMSTRUCT*)lp;
			COLORREF cr = (COLORREF)dis.itemData;
			if(cr == CB_ERR)
				return FALSE;
			
			int nSave = SaveDC(dis.hDC);
			COLORREF crBack = GetSysColor((dis.itemState & ODS_DISABLED) ? COLOR_BTNFACE
				: (dis.itemState & ODS_FOCUS) ? COLOR_HIGHLIGHT : COLOR_WINDOW);
			COLORREF crText = GetSysColor((dis.itemState & ODS_DISABLED) ? COLOR_GRAYTEXT
				: (dis.itemState & ODS_FOCUS) ? COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT);
			
			COLORREF crOld = SetTextColor(dis.hDC, crText);
			int nOldBk = SetBkMode(dis.hDC, TRANSPARENT);
			
			HBRUSH hbrBack = CreateSolidBrush(crBack);
			FillRect(dis.hDC, &dis.rcItem, hbrBack);
			
			HBRUSH hbrColor = CreateSolidBrush(cr);
			SelectObject(dis.hDC, hbrColor);
			SelectObject(dis.hDC, GetStockObject(BLACK_PEN));
			Rectangle(dis.hDC, dis.rcItem.left + 2, dis.rcItem.top + 2,
				dis.rcItem.left + 24, dis.rcItem.bottom - 2);
			
			RECT rc = dis.rcItem;
			int len = SendMessage(dis.hwndItem, CB_GETLBTEXTLEN, dis.itemID, 0);
			char *str = (char*)_alloca(len + 1);
			SendMessage(dis.hwndItem, CB_GETLBTEXT, dis.itemID, (LPARAM)str);
			rc.left += 28;
			DrawText(dis.hDC, str, len, &rc, DT_SINGLELINE | DT_LEFT | DT_VCENTER | DT_NOPREFIX);
			
			if(dis.itemState & ODS_FOCUS)
			{
				if(dis.itemState & ODS_COMBOBOXEDIT)
					DrawFocusRect(dis.hDC, &dis.rcItem);
				else
					// WINBUG: DrawFocusRect works incorrectly sometimes
					PseudoDrawFocusRect(dis.hDC, &dis.rcItem);
			}
			RestoreDC(dis.hDC, nSave);
			DeleteObject(hbrBack);
			DeleteObject(hbrColor);
			return TRUE;
		}
		case WM_SYSCOLORCHANGE:
		{
			colors[0] = GetSysColor(COLOR_WINDOWTEXT);
			SendDlgItemMessage(hwnd, IDC_COLORCOMBO, CB_SETITEMDATA, 0, colors[0]);
			return FALSE;
		}
	}
	return FALSE;
}

// Mark "More" button: if filter enabled, "*More..."
void MarkMoreButton(HWND hwnd, UINT uID, const FILTERINFO& fi)
{
	char str[16];
	LPSTR str2;
	GetDlgItemText(hwnd, uID, &str[1], 14);
	str[0] = '*';
	if(fi.v.size() == 0)
		str2 = (str[1] == '*') ? &str[2] : &str[1];
	else
		str2 = (str[1] == '*') ? &str[1] : str;
	SetDlgItemText(hwnd, uID, str2);
}

inline BOOL FilterText(const String str, const FILTERTEXT& ft)
{
	return (StrStrI(str, ft.text) ? 1 : 0) ^ (BYTE&)ft.bNot;
}

// AdvancedFilter returns if mail is target
bool AdvancedFilter(const INDEXINFO& idxinfo, const FILTERINFO& fi)
{
	for(FILTERINFO::TList::const_iterator it = fi.v.begin(); it != fi.v.end(); ++it)
	{
		DWORD dwFlag = it->dwFlag;
		BOOL b = 1;
		if(dwFlag & FILTER_FROM)
			b &= FilterText(idxinfo.from, it->from);
		if(dwFlag & FILTER_TO)
			b &= FilterText(idxinfo.to, it->to);
		if(dwFlag & FILTER_SUBJECT)
			b &= FilterText(idxinfo.subject, it->subject);
		if(dwFlag & FILTER_PRIORITY)
			if(it->nPriorityUnit == 0)
				b &= (idxinfo.nPriority == it->nPriority + 1);
			else if(it->nPriorityUnit == 1)
				b &= (idxinfo.nPriority <= it->nPriority + 1);
			else
				b &= (idxinfo.nPriority >= it->nPriority + 1);
		if(dwFlag & FILTER_COLORLABEL)
			b &= (idxinfo.color == it->color);
		if(dwFlag & FILTER_STATUS)
			b &= ((idxinfo.dwStatus & 0x00000001) != it->nStatus);
		if(dwFlag & FILTER_SIZE)
			if(it->nSizeUnit == 0)
				b &= (idxinfo.dwSize > it->nSize);
			else
				b &= (idxinfo.dwDate < it->nSize);
		if(b)
			return true;
	}
	return false;
}
