// foldercombo.cpp - Implement folder combobox
//
// Old Mail Arrangement Plug-in for Becky! 2
// Copyright (C) 2000 by Tietew
//
#include "pch.h"
#include "bkrmold.h"
#include "foldercombo.h"
#include "utys.h"

// IDs of special folder's description in string table
#define BKIDS_FOLDER_INBOX		85
#define BKIDS_FOLDER_OUTBOX		86
#define BKIDS_FOLDER_DRAFT		87
#define BKIDS_FOLDER_SENT		88
#define BKIDS_FOLDER_REMINDER	89
#define BKIDS_FOLDER_TRASH		90

// Indecies of folder icons in the imagelist
#define BKIDX_FOLDER_MB_POP3	5
#define BKIDX_FOLDER_MB_IMAP4	9
#define BKIDX_FOLDER_INBOX		13
#define BKIDX_FOLDER_OUTBOX		15
#define BKIDX_FOLDER_DRAFT		17
#define BKIDX_FOLDER_SENT		19
#define BKIDX_FOLDER_REMINDER	21
#define BKIDX_FOLDER_TRASH		23
#define BKIDX_FOLDER_COLOR0		31
#define BKIDX_FOLDER_COLOR1		25
#define BKIDX_FOLDER_COLOR2		27
#define BKIDX_FOLDER_COLOR3		29
#define BKIDX_FOLDER_COLOR4		33

// Map of special folders
static struct {
	char *name;
	UINT id;
	UINT icon;
} SpecialFolders[] = {
	{ "!!!!InBox", BKIDS_FOLDER_INBOX, BKIDX_FOLDER_INBOX },
	{ "!!!!OutBox", BKIDS_FOLDER_OUTBOX, BKIDX_FOLDER_OUTBOX },
	{ "!!!Draft", BKIDS_FOLDER_DRAFT, BKIDX_FOLDER_DRAFT },
	{ "!!!Sent", BKIDS_FOLDER_SENT, BKIDX_FOLDER_SENT },
	{ "!!!XReminder", BKIDS_FOLDER_REMINDER, BKIDX_FOLDER_REMINDER },
	{ "!Trash", BKIDS_FOLDER_TRASH, BKIDX_FOLDER_TRASH },
};
// color - index map
static UINT IconColors[] = {
	BKIDX_FOLDER_COLOR0,
	BKIDX_FOLDER_COLOR1,
	BKIDX_FOLDER_COLOR2,
	BKIDX_FOLDER_COLOR3,
	BKIDX_FOLDER_COLOR4,
};

// Forward reference
static bool ProcessFolders(HWND hCombo, String path, int idxmb,
	String cur, int indent, HANDLE hItem, INSERTCALLBACK pfnInsert, DWORD dwContext);

// Add item to combobox
static void ProcessFolder(HWND hCombo, String fname, String path, int idxmb,
	String cur, int indent, HANDLE hItem, INSERTCALLBACK pfnInsert, DWORD dwContext)
{
	String name, path2 = path + fname;
	int icon = -1;
	
	if(fname[0] == '!')		// Special folders
	{
		// Get description from string table of Becky!
		for(int i = 0; i < countof(SpecialFolders); ++i)
			if(lstrcmpi(fname, SpecialFolders[i].name) == 0)
			{
				char name2[MAX_PATH];
				LoadString(bka.GetB2Instance(), SpecialFolders[i].id, name2, MAX_PATH);
				name = name2;
				icon = SpecialFolders[i].icon;
				break;
			}
	}
	else if(fname[0] == '#')	// Template folder
		return;
	
	path2 += '\\';
	if(name == String::Nil)
	{
		// Get color of folder
		int color = GetPrivateProfileInt("Settings", "Color", 0, String(path2 + "Folder.ini"));
		if(color < 0 || color >= countof(IconColors))
			color = 0;
		icon = IconColors[color];
		name = fname;
	}
	
	// Allocate structure and store informations
	INSERTINFO ii;
	ii.fi.indent = indent;
	ii.fi.icon = icon;
	ii.fi.desc = name;
	ii.fi.display = bka.GetFolderDisplayName(&path2[idxmb]);
	ii.fi.identifier = &path2[idxmb];
	ii.hCtrl = hCombo;
	ii.hParent = hItem;
	ii.bCurrent = lstrcmpi(cur, &path2[idxmb]) == 0;
	hItem = (*pfnInsert)(ii, dwContext);
	
	// Process sub folders
	if(icon != BKIDX_FOLDER_TRASH)
		ProcessFolders(hCombo, path2, idxmb, cur, indent + 1, hItem, pfnInsert, dwContext);
}

// Scan a folder and process each folder
bool ProcessFolders(HWND hCombo, String path, int idxmb, String cur,
	int indent, HANDLE hItem, INSERTCALLBACK pfnInsert, DWORD dwContext)
{
	WIN32_FIND_DATA fd;
	vector<String> v;
	HANDLE hFind = FindFirstFile(String(path + '*'), &fd);
	if(hFind != INVALID_HANDLE_VALUE)
	{
		// Scan and make vector
		do
			if(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				if(lstrcmp(fd.cFileName, ".") == 0 || lstrcmp(fd.cFileName, "..") == 0)
					continue;
				v.push_back(fd.cFileName);
			}
		while(FindNextFile(hFind, &fd));
		FindClose(hFind);
		
		// Add items in ASCII order
		sort(v.begin(), v.end());
		for(vector<String>::iterator it = v.begin(); it != v.end(); ++it)
			ProcessFolder(hCombo, *it, path, idxmb, cur, indent, hItem, pfnInsert, dwContext);
	}
	return hFind != INVALID_HANDLE_VALUE;
}

// Initialize folder combobox
bool InitFolderCtrlEx(HWND hCtrl, String cur, INSERTCALLBACK pfnInsert, DWORD dwContext)
{
	String path = bka.GetDataFolder();
	int idxmb = path.length();
	bool bSuccess = false;
	
	// Scan all mailboxes
	vector<String> v;
	WIN32_FIND_DATA fd;
	HANDLE hFind = FindFirstFile(String(path + "*.mb"), &fd);
	if(hFind != INVALID_HANDLE_VALUE)
	{
		do
			if((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
				v.push_back(fd.cFileName);
		while(FindNextFile(hFind, &fd));
		bSuccess = true;
	}
	FindClose(hFind);
	
	sort(v.begin(), v.end());
	for(vector<String>::iterator it = v.begin(); it != v.end(); ++it)
	{
		String id = *it + '\\';
		String mb = path + id;
		String mbname = bka.GetFolderDisplayName(id);
		
		// Add mailbox item
		INSERTINFO ii;
		ii.fi.indent = 0;
		ii.fi.icon = GetPrivateProfileInt("Account", "Protocol", 0, String(mb + "Mailbox.ini"))
			? BKIDX_FOLDER_MB_IMAP4 : BKIDX_FOLDER_MB_POP3;
		ii.fi.desc = mbname;
		ii.fi.display = mbname;
		ii.fi.identifier = id;
		ii.hCtrl = hCtrl;
		ii.hParent = NULL;
		ii.bCurrent = FALSE;
		HANDLE hItem = (*pfnInsert)(ii, dwContext);
		
		// Process all sub folders
		if(!ProcessFolders(hCtrl, mb, idxmb, cur, 1, hItem, pfnInsert, dwContext))
		{
			bSuccess = false;
			break;
		}
	}
	
	if(!bSuccess)
		MyMessageBox(GetParent(hCtrl), IDS_FAIL_ENUMFOLDER, MB_OK | MB_ICONSTOP);
	
	return bSuccess;
}

struct FOLDERCOMBOINSERT
{
	HDC hdc;
	int maxwidth;
};

static HANDLE ComboInsertCallback(const INSERTINFO& ii, DWORD dwContext)
{
	FOLDERCOMBOINSERT& fci = *(FOLDERCOMBOINSERT*)dwContext;
	SIZE size;
	GetTextExtentPoint32(fci.hdc, ii.fi.desc, ii.fi.desc.length(), &size);
	size.cx += ii.fi.indent * 16 + 16 + 6;
	if(fci.maxwidth < size.cx)
		fci.maxwidth = size.cx;
	FOLDERINFO *pFI = new FOLDERINFO(ii.fi);
	if(!ii.hParent)
		pFI->identifier = String::Nil;
	int idx = SendMessage(ii.hCtrl, CB_ADDSTRING, 0, (LPARAM)pFI);
	if(ii.bCurrent)
		SendMessage(ii.hCtrl, CB_SETCURSEL, idx, 0);
	return (HANDLE)0xFFFFFFFF;
}

bool InitFolderCombo(HWND hdlg, UINT id, String cur)
{
	HWND hCombo = GetDlgItem(hdlg, id);
	LOGFONT lf;
	if(!SystemParametersInfo(SPI_GETICONTITLELOGFONT, sizeof lf, &lf, 0))
		return false;
	HFONT hFont = CreateFontIndirect(&lf);
	if(!hFont)
		return false;
	SendMessage(hCombo, WM_SETFONT, (WPARAM)hFont, FALSE);
	
	HDC hdc = GetDC(hCombo);
	HGDIOBJ hFontOld = SelectObject(hdc, hFont);
	FOLDERCOMBOINSERT fci = { hdc, 0 };
	bool bSuccess = InitFolderCtrlEx(hCombo, cur,
		ComboInsertCallback, (DWORD)&fci);
	if(bSuccess)
	{
		SendMessage(hCombo, CB_SETDROPPEDWIDTH,
			fci.maxwidth + GetSystemMetrics(SM_CXVSCROLL) + GetSystemMetrics(SM_CXBORDER) * 2, 0);
	}
	SelectObject(hdc, hFontOld);
	ReleaseDC(hCombo, hdc);
	
	return bSuccess;
}

// Teminate folder combobox
void TermFolderCombo(HWND hdlg, UINT id)
{
	HWND hCombo = GetDlgItem(hdlg, id);
	int max = SendMessage(hCombo, CB_GETCOUNT, 0, 0);
	// Release all structures
	for(int i = 0; i < max; ++i)
		delete (FOLDERINFO*)SendMessage(hCombo, CB_GETITEMDATA, i, 0);
	DeleteObject((HFONT)SendMessage(hCombo, WM_GETFONT, 0, 0));
}

// Process WM_MEASUREITEM and WM_DRAWITEM
LRESULT ProcessFolderCombo(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	if(msg == WM_MEASUREITEM)
	{
		// Height of item is max of icon and font height
		TEXTMETRIC tm;
		HFONT hFont = (HFONT)SendMessage(hwnd, WM_GETFONT, 0, 0);
		HFONT hFontOld;
		HDC hdc = GetDC(hwnd);
		if(hFont)
			hFontOld = (HFONT)SelectObject(hdc, hFont);
		GetTextMetrics(hdc, &tm);
		if(hFont)
			SelectObject(hdc, hFontOld);
		ReleaseDC(hwnd, hdc);
		int height = tm.tmHeight + tm.tmInternalLeading + 2;
		if(height < 16)
			height = 16;
		((MEASUREITEMSTRUCT*)lp)->itemHeight = height;
	}
	else if(msg == WM_DRAWITEM)
	{
		// Draw a combobox item
		DRAWITEMSTRUCT& dis = *(DRAWITEMSTRUCT*)lp;
		FOLDERINFO *fi = (FOLDERINFO*)dis.itemData;
		if(IsBadReadPtr(fi, sizeof FOLDERINFO))
			return 0;
		int nSave = SaveDC(dis.hDC);
		
		// colors and brushes
		COLORREF crWindow = GetSysColor((dis.itemState & ODS_DISABLED) ? COLOR_BTNFACE : COLOR_WINDOW);
		COLORREF crBase = (dis.itemState & ODS_DISABLED) ? crWindow :
			GetSysColor((dis.itemState & ODS_SELECTED) ? COLOR_HIGHLIGHT : COLOR_WINDOW);
		COLORREF crText = GetSysColor((dis.itemState & ODS_DISABLED) ? COLOR_GRAYTEXT :
			((dis.itemState & ODS_SELECTED) ? COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT));
		HBRUSH hbrWindow = CreateSolidBrush(crWindow);
		HBRUSH hbrBase = CreateSolidBrush(crBase);
		
		// DC modes
		SetBkMode(dis.hDC, TRANSPARENT);
		SetTextColor(dis.hDC, crText);
		
		if(dis.itemState & ODS_COMBOBOXEDIT)
		{
			// Combobox edit area
			FillRect(dis.hDC, &dis.rcItem, hbrBase);
			if(dis.itemState & ODS_DISABLED)
			{
				// disabled ... draw disabled icon
				HICON hIcon = ImageList_GetIcon(g_hilFolders, fi->icon, ILD_TRANSPARENT);
				DrawState(dis.hDC, NULL, NULL, (LPARAM)hIcon, 0,
					dis.rcItem.left, dis.rcItem.top, 0, 0, DST_ICON | DSS_DISABLED);
				DestroyIcon(hIcon);
			}
			else
				ImageList_Draw(g_hilFolders, fi->icon, dis.hDC, dis.rcItem.left, dis.rcItem.top,
					((dis.itemState & ODS_FOCUS) ? ILD_FOCUS : 0) | ILD_TRANSPARENT);
			// draw text and focus rect
			RECT rc = dis.rcItem;
			rc.left += 16 + 1;
			rc.right -= 1;
			rc.top += 1;
			rc.bottom -= 1;
			DrawText(dis.hDC, fi->display, fi->display.length(), &rc,
				DT_SINGLELINE | DT_NOPREFIX | DT_LEFT | DT_VCENTER);
			if(dis.itemState & ODS_FOCUS)
				DrawFocusRect(dis.hDC, &dis.rcItem);
		}
		else
		{
			// draw a item in drop-down list
			SIZE size;
			// Calculate sizes
			RECT rcLeft = dis.rcItem;
			RECT rcText = dis.rcItem;
			RECT rcRight = dis.rcItem;
			GetTextExtentPoint32(dis.hDC, fi->desc, fi->desc.length(), &size);
			rcLeft.right = (fi->indent + 1) * 16 + 2;
			rcText.left = rcLeft.right;
			rcText.right = rcText.left + size.cx + 2 + 2;
			rcRight.left = rcText.right;
			// Fill backgrounds
			FillRect(dis.hDC, &rcLeft, hbrWindow);
			FillRect(dis.hDC, &rcText, hbrBase);
			FillRect(dis.hDC, &rcRight, hbrWindow);
			// icon, text, and focus rect
			ImageList_Draw(g_hilFolders, (dis.itemState & ODS_SELECTED) ? fi->icon - 1 : fi->icon, dis.hDC,
				rcText.left - 16 - 1, dis.rcItem.top + 1,
				((dis.itemState & ODS_SELECTED) ? ILD_FOCUS : 0) | ILD_TRANSPARENT);
			rcText.left += 2;
			DrawText(dis.hDC, fi->desc, fi->desc.length(), &rcText,
				DT_SINGLELINE | DT_NOPREFIX | DT_LEFT | DT_VCENTER);
			rcText.left -= 2;
			if((dis.itemState & ODS_SELECTED))
				// WINBUG: DrawFocusRect works incorrectly sometimes
				PseudoDrawFocusRect(dis.hDC, &rcText);
		}
		RestoreDC(dis.hDC, nSave);
		
		DeleteObject(hbrBase);
		DeleteObject(hbrWindow);
	}
	return 0;
}

// Get identifier of selected folder
String GetSelectedFolder(HWND hwnd, UINT id)
{
	FOLDERINFO *fi = (FOLDERINFO*)SendDlgItemMessage(hwnd, id, CB_GETITEMDATA,
		SendDlgItemMessage(hwnd, id, CB_GETCURSEL, 0, 0), 0);
	return IsBadReadPtr(fi, sizeof FOLDERINFO) ? String::Nil : fi->identifier;
}
