// progress.cpp - Implements progress window
//
// Old Mail Arrangement Plug-in for Becky! 2
// Copyright (C) 2000 by Tietew
//
#include "pch.h"
#include "bkrmold.h"
#include "progress.h"

// constructor
CProgress::CProgress(bool bUI)
	: m_hDlg(NULL), m_bUI(bUI), m_bTerm(false), m_nLastPercent(0)
{
	// Disables main window and set controls not to redraw
	DisableUI();
	if(bUI)
	{
		// Create the dialog
		m_hDlg = CreateDialogParam(g_hInstance, MAKEINTRESOURCE(IDD_PROGRESS),
			m_wnd.hwndMain, (DLGPROC)ProgressProc, (LPARAM)&m_bTerm);
		ShowWindow(m_hDlg, SW_SHOW);
	}
}

// destructor
CProgress::~CProgress()
{
	if(m_bUI)
	{
		// Enables main window and set controls to redraw
		SendMessage(m_wnd.hwndListHead, WM_SETREDRAW, TRUE, 0);
		SendMessage(m_wnd.hwndStatus, WM_SETREDRAW, TRUE, 0);
		SendMessage(m_wnd.hwndList, WM_SETREDRAW, TRUE, 0);
		EnableWindow(m_wnd.hwndMain, TRUE);
	}
	// Destroy the dialog
	if(m_hDlg)
		DestroyWindow(m_hDlg);
	// Force redraw controls
	InvalidateRect(m_wnd.hwndMain, NULL, FALSE);
	UpdateWindow(m_wnd.hwndMain);
}

// Disables main window and set controls not to redraw
void CProgress::DisableUI()
{
	EnableWindow(m_wnd.hwndMain, FALSE);
	SendMessage(m_wnd.hwndList, WM_SETREDRAW, FALSE, 0);
	SendMessage(m_wnd.hwndStatus, WM_SETREDRAW, FALSE, 0);
	SendMessage(m_wnd.hwndListHead, WM_SETREDRAW, FALSE, 0);
}

// Set percentage and pump messages
bool CProgress::DispatchMessage(int n, int nPercent)
{
	if(m_hDlg)
	{
		// Pump messages
		MSG msg;
		while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
			if(!IsDialogMessage(m_hDlg, &msg))
				::DispatchMessage(&msg);
		// If the percentage changes, set it
		if(nPercent != m_nLastPercent)
		{
			SendMessage(m_hDlg, WM_USER, (WPARAM)nPercent, n);
			m_nLastPercent = nPercent;
		}
	}
	// Returns wether CANCEL button pushed
	return m_bTerm;
}

// dialog procedure
BOOL CALLBACK CProgress::ProgressProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			// Remember pointer to cancel flag
			SetWindowLong(hwnd, DWL_USER, lp);
			// Initialize controls
			SendDlgItemMessage(hwnd, IDC_PROGRESS1, PBM_SETRANGE, 0, MAKELPARAM(0, 100));
			SendDlgItemMessage(hwnd, IDC_PROGRESS1, PBM_SETPOS, 0, 0);
			SendDlgItemMessage(hwnd, IDC_PROGRESS2, PBM_SETRANGE, 0, MAKELPARAM(0, 100));
			SendDlgItemMessage(hwnd, IDC_PROGRESS2, PBM_SETPOS, 0, 0);
			// Disables close button
			EnableMenuItem(GetSystemMenu(hwnd, FALSE), SC_CLOSE, MF_DISABLED | MF_BYCOMMAND);
			return TRUE;
		}
		case WM_COMMAND:
			// cancel
			if(wp == IDCANCEL)
				*(bool*)GetWindowLong(hwnd, DWL_USER) = true;
			break;
		
		case WM_USER:
		{
			// Set progress bar position
			char perc[6];
			wsprintf(perc, "%d%%", wp);
			SetDlgItemText(hwnd, lp ? IDC_PERCENT2 : IDC_PERCENT1, perc);
			SendDlgItemMessage(hwnd, lp ? IDC_PROGRESS2 : IDC_PROGRESS1, PBM_SETPOS, wp, 0);
			return TRUE;
		}
		case WM_NCHITTEST:
			// All area of the dialog is client
			SetWindowLong(hwnd, DWL_MSGRESULT, HTCLIENT);
			return TRUE;
		
		// Eat messages
		case WM_CONTEXTMENU:
		case WM_NCRBUTTONDOWN:
		case WM_RBUTTONDOWN:
		case WM_SYSKEYDOWN:
		case WM_SYSKEYUP:
		case WM_SYSCOMMAND:
			return TRUE;
	}
	return FALSE;
}
