// schedule.cpp - shcedule
//
// Old Mail Arrangement Plug-in for Becky! 2
// Copyright (C) 2000 by Tietew
//
#include "pch.h"
#include "bkrmold.h"
#include "remove.h"
#include "schedule.h"
#include "utys.h"
#include "foldercombo.h"

static unsigned __int64 hex64(LPCSTR str)
{
	unsigned __int64 r = 0;
	while(*str && (*str >= '0' && *str <= '9' || *str >= 'A' && *str <= 'F' || *str >= 'a' && *str <= 'f'))
	{
		r <<= 4;
		r |= *str >= 'a' ? *str - 'a' + 10 : (*str >= 'A' ? *str - 'A' + 10 : *str - '0');
		++str;
	}
	return r;
}
static void hex64(LPSTR str, unsigned __int64 hex)
{
	for(int i = 15; i >= 0; --i)
	{
		str[i] = "0123456789ABCDEF"[hex & 0xF];
		hex >>= 4;
	}
	str[16] = '\0';
}

// Read schedule settings from Folder.ini
void GetScheduleInfo(String folder, SCHEDULEINFO& si)
{
	String path = bka.GetDataFolder() + folder + "Folder.ini";
	si.bEnable = GetPrivateProfileInt(INI_SECTION, "Enable", FALSE, path);
	si.nTrigger = GetPrivateProfileInt(INI_SECTION, "Trigger", 0, path) ? IDC_ONEND : IDC_ONSELECT;
	si.nFMails = GetPrivateProfileInt(INI_SECTION, "FMails", GetIniMails(), path);
	si.nFilter = GetPrivateProfileInt(INI_SECTION, "Filter", GetIniFilter(), path) ? IDC_FLAST : IDC_FDAYS;
	si.nMails = GetPrivateProfileInt(INI_SECTION, "Mails", GetIniMails(), path);
	si.nDays = GetPrivateProfileInt(INI_SECTION, "Days", GetIniDays(), path);
	si.bThreads = GetPrivateProfileInt(INI_SECTION, "Threads", 0, path);
	si.nBehavior = GetPrivateProfileInt(INI_SECTION, "Behavior", 0, path) ? IDC_MOVE : IDC_REMOVE;
	si.bRecursive = GetPrivateProfileInt(INI_SECTION, "Recursive", FALSE, path);
	char dest[MAX_PATH];
	GetPrivateProfileString(INI_SECTION, "Destination", "", dest, countof(dest), path);
	si.target = dest;
	si.bLockout = GetPrivateProfileInt(INI_SECTION, "Lockout", FALSE, path);
	si.nLockoutNum = GetPrivateProfileInt(INI_SECTION, "LockoutNum", 0, path);
	si.nLockoutUnit = GetPrivateProfileInt(INI_SECTION, "LockoutUnit", 0, path);
	if(si.nLockoutUnit < 0 || si.nLockoutUnit > 2)
		si.nLockoutUnit = 0;
	int max = GetPrivateProfileInt(INI_SECTION, "Filters", 0, path);
	for(int i = 0; i < max; ++i)
		GetFilterItem(si.filter, path, i);
}

// Write schedule settings to Folder.ini
void SetScheduleInfo(String folder, const SCHEDULEINFO& si, bool bRecursive)
{
	String path = bka.GetDataFolder() + folder;
	if(bRecursive)
	{
		// Apply settings to sub folders
		WIN32_FIND_DATA fd;
		HANDLE hFind = FindFirstFile(String(path + '*'), &fd);
		if(hFind != INVALID_HANDLE_VALUE)
		{
			do
			{
				if(lstrcmp(fd.cFileName, ".") == 0 || lstrcmp(fd.cFileName, "..") == 0)
					continue;
				if(fd.cFileName[0] == '#')
					continue;
				if(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
					SetScheduleInfo(folder + fd.cFileName + '\\', si, true);
			}
			while(FindNextFile(hFind, &fd));
			FindClose(hFind);
		}
	}
	path += "Folder.ini";
	WriteIniInt(INI_SECTION, "Enable", si.bEnable, path);
	WriteIniInt(INI_SECTION, "Trigger", si.nTrigger == IDC_ONEND, path);
	WriteIniInt(INI_SECTION, "FMails", si.nFMails, path);
	WriteIniInt(INI_SECTION, "Filter", si.nFilter == IDC_FLAST, path);
	WriteIniInt(INI_SECTION, "Mails", si.nMails, path);
	WriteIniInt(INI_SECTION, "Days", si.nDays, path);
	WriteIniInt(INI_SECTION, "Threads", si.bThreads, path);
	WriteIniInt(INI_SECTION, "Behavior", si.nBehavior == IDC_MOVE, path);
	WriteIniInt(INI_SECTION, "Recursive", si.bRecursive, path);
	WritePrivateProfileString(INI_SECTION, "Destination", si.target, path);
	WriteIniInt(INI_SECTION, "Lockout", si.bLockout, path);
	WriteIniInt(INI_SECTION, "LockoutNum", si.nLockoutNum, path);
	WriteIniInt(INI_SECTION, "LockoutUnit", si.nLockoutUnit, path);
	int i = 0;
	for(FILTERINFO::TList::const_iterator it = si.filter.v.begin(); it != si.filter.v.end(); ++it)
		if(it->dwFlag != 0)
			SetFilterItem(*it, i++, path);
	WriteIniInt(INI_SECTION, "Filters", i, path);
}

// Dialog procedure
BOOL CALLBACK ScheduleProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			pair<String, FILTERINFO> *pData = new pair<String, FILTERINFO>;
			String path = (LPCSTR)lp;
			pData->first = path;
			SetWindowLong(hwnd, DWL_USER, (LONG)pData);
			
			SCHEDULEINFO si;
			GetScheduleInfo(path, si);
			
			// Initialize controls
			SetDlgItemText(hwnd, IDC_FOLDER, bka.GetFolderDisplayName(path));
			SendDlgItemMessage(hwnd, IDC_DAYSSPIN, UDM_SETRANGE32, 0, INT_MAX);
			SendDlgItemMessage(hwnd, IDC_MAILSSPIN, UDM_SETRANGE32, 0, INT_MAX);
			SendDlgItemMessage(hwnd, IDC_FMAILSSPIN, UDM_SETRANGE32, 0, INT_MAX);
			SendDlgItemMessage(hwnd, IDC_LOCKOUTSPIN, UDM_SETRANGE32, 0, INT_MAX);
			CheckDlgButton2(hwnd, IDC_ENABLE, si.bEnable);
			CheckRadioButton(hwnd, IDC_ONSELECT, IDC_ONEND, si.nTrigger);
			CheckRadioButton(hwnd, IDC_FDAYS, IDC_FLAST, si.nFilter);
			SetDlgItemInt(hwnd, IDC_FMAILS, si.nFMails, FALSE);
			SetDlgItemInt(hwnd, IDC_DAYS, si.nDays, FALSE);
			SetDlgItemInt(hwnd, IDC_MAILS, si.nMails, FALSE);
			CheckDlgButton2(hwnd, IDC_THREAD, si.bThreads);
			CheckRadioButton(hwnd, IDC_REMOVE, IDC_MOVE, si.nBehavior);
			CheckDlgButton2(hwnd, IDC_RECURSIVE, si.bRecursive);
			char unit[16];
			for(UINT i = IDS_UNIT1; i <= IDS_UNIT3; ++i)
			{
				LoadString(g_hInstance, i, unit, countof(unit));
				SendDlgItemMessage(hwnd, IDC_LOCKOUTUNIT, CB_ADDSTRING, 0, (LPARAM)unit);
			}
			CheckDlgButton2(hwnd, IDC_LOCKOUT, si.bLockout);
			SetDlgItemInt(hwnd, IDC_LOCKOUTNUM, si.nLockoutNum, FALSE);
			SendDlgItemMessage(hwnd, IDC_LOCKOUTUNIT, CB_SETCURSEL, si.nLockoutUnit, 0);
			pData->second = si.filter;
			MarkMoreButton(hwnd, IDC_MORE, si.filter);
			
			// Initialize folder combobox
			if(!InitFolderCombo(hwnd, IDC_MOVETO, si.target[0] ? si.target : bka.GetCurrentFolder()))
				EndDialog(hwnd, IDCANCEL);
			PostMessage(hwnd, WM_COMMAND, IDC_ENABLE, 0);
			return TRUE;
		}
		case WM_COMMAND:
			switch(LOWORD(wp))
			{
				case IDC_ONEND:
				case IDC_ONSELECT:
				case IDC_MOVE:
				case IDC_REMOVE:
				case IDC_LOCKOUT:
				case IDC_ENABLE:
				{
					// Enableness of controls
					bool bEnable = IsDlgButtonChecked2(hwnd, IDC_ENABLE);
					bool eFMails = bEnable && IsDlgButtonChecked2(hwnd, IDC_ONSELECT);
					bool eDest = bEnable && IsDlgButtonChecked2(hwnd, IDC_MOVE);
					bool eLock = bEnable && IsDlgButtonChecked2(hwnd, IDC_LOCKOUT);
					// Alert
					if((LOWORD(wp) == IDC_ONEND || LOWORD(wp) == IDC_ENABLE) && bEnable && !eFMails && !GetIniExitSchedule())
						MyMessageBox(hwnd, IDS_EXITSCHEDULE_DISABLED, MB_OK | MB_ICONINFORMATION);
					EnableWindow(GetDlgItem(hwnd, IDC_FMAILS), eFMails);
					EnableWindow(GetDlgItem(hwnd, IDC_STATIC_FMAILS), eFMails);
					EnableWindow(GetDlgItem(hwnd, IDC_STATIC_FMAILS2), eFMails);
					EnableWindow(GetDlgItem(hwnd, IDC_STATIC_MOVETO), eDest);
					EnableWindow(GetDlgItem(hwnd, IDC_MOVETO), eDest);
					EnableWindow(GetDlgItem(hwnd, IDC_LOCKOUTNUM), eLock);
					EnableWindow(GetDlgItem(hwnd, IDC_LOCKOUTUNIT), eLock);
					static UINT ids[] = {
						IDC_STATIC_TRIGGER, IDC_ONSELECT, IDC_ONEND, IDC_STATIC_FILTER,
						IDC_STATIC_DAYS, IDC_DAYS, IDC_STATIC_DAYS2,
						IDC_STATIC_MAILS, IDC_MAILS, IDC_STATIC_MAILS2,
						IDC_FDAYS, IDC_FLAST, IDC_THREAD, IDC_MORE,
						IDC_STATIC_BEHAVIOR, IDC_MOVE, IDC_REMOVE, IDC_RECURSIVE,
						IDC_LOCKOUT,
					};
					for(int i = 0; i < countof(ids); ++i)
						EnableWindow(GetDlgItem(hwnd, ids[i]), bEnable);
					return TRUE;
				}
				case IDC_DAYS:
					if(HIWORD(wp) == EN_SETFOCUS)
						CheckRadioButton(hwnd, IDC_FDAYS, IDC_FLAST, IDC_FDAYS);
					return TRUE;
				
				case IDC_MAILS:
					if(HIWORD(wp) == EN_SETFOCUS)
						CheckRadioButton(hwnd, IDC_FDAYS, IDC_FLAST, IDC_FLAST);
					return TRUE;
				
				case IDC_MORE:
				{
					pair<String, FILTERINFO> *pData;
					(LONG&)pData = GetWindowLong(hwnd, DWL_USER);
					FILTERDLGINFO fdi;
					fdi.fi = pData->second;
					fdi.nCurSel = 0;
					int result = DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_FILTER),
						hwnd, (DLGPROC)FilterProc, (LPARAM)&fdi);
					if(result == IDOK)
						pData->second = fdi.fi;
					MarkMoreButton(hwnd, IDC_MORE, pData->second);
					return TRUE;
				}
				case IDOK:
				{
					pair<String, FILTERINFO> *pData;
					(LONG&)pData = GetWindowLong(hwnd, DWL_USER);
					// Apply settings
					SCHEDULEINFO si;
					si.bEnable = IsDlgButtonChecked2(hwnd, IDC_ENABLE);
					si.nTrigger = IsDlgButtonChecked2(hwnd, IDC_ONEND) ? IDC_ONEND : IDC_ONSELECT;
					si.nFMails = GetDlgItemInt(hwnd, IDC_FMAILS, NULL, FALSE);
					si.nFilter = IsDlgButtonChecked2(hwnd, IDC_FLAST) ? IDC_FLAST : IDC_FDAYS;
					si.nMails = GetDlgItemInt(hwnd, IDC_MAILS, NULL, FALSE);
					si.nDays = GetDlgItemInt(hwnd, IDC_DAYS, NULL, FALSE);
					si.bThreads = IsDlgButtonChecked2(hwnd, IDC_THREAD);
					si.nBehavior = IsDlgButtonChecked2(hwnd, IDC_MOVE) ? IDC_MOVE : IDC_REMOVE;
					si.bRecursive = IsDlgButtonChecked2(hwnd, IDC_RECURSIVE);
					si.target = GetSelectedFolder(hwnd, IDC_MOVETO);
					if(si.nBehavior == IDC_MOVE && (pData->first == si.target || si.target == String::Nil))
					{
						MyMessageBox(hwnd, si.target == String::Nil ? IDS_MOVETO_MAILBOX : IDS_MOVETO_ME,
							MB_OK | MB_ICONEXCLAMATION);
						SetFocus(GetDlgItem(hwnd, IDC_MOVETO));
						return TRUE;
					}
					si.bLockout = IsDlgButtonChecked2(hwnd, IDC_LOCKOUT);
					si.nLockoutNum = GetDlgItemInt(hwnd, IDC_LOCKOUTNUM, NULL, FALSE);
					si.nLockoutUnit = SendDlgItemMessage(hwnd, IDC_LOCKOUTUNIT, CB_GETCURSEL, 0, 0);
					si.filter = pData->second;
					SetScheduleInfo(pData->first, si, IsDlgButtonChecked2(hwnd, IDC_INHERIT));
				}
				case IDCANCEL:
					EndDialog(hwnd, wp);
					return TRUE;
			}
			break;
		
		case WM_MEASUREITEM:
		case WM_DRAWITEM:
			return ProcessFolderCombo(hwnd, msg, wp, lp);
		
		case WM_DESTROY:
			TermFolderCombo(hwnd, IDC_MOVETO);
			delete (pair<String, FILTERINFO>*)GetWindowLong(hwnd, DWL_USER);
			break;
	}
	return FALSE;
}

// Menu handler of schedule
void CALLBACK OnSchedule(HWND hWnd, LPARAM lParam)
{
	DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_SCHEDULE), hWnd,
		(DLGPROC)ScheduleProc, (LPARAM)bka.GetCurrentFolder());
}

// Menu handler of schedule
UINT CALLBACK OnUpdateSchedule(HWND hWnd, LPARAM lParam)
{
	UINT res = BKMENU_CMDUI_DISABLED;
	String cf = bka.GetCurrentFolder();
	if(lstrcmpi(bka.GetCurrentMailBox(), cf) != 0)
	{
		// Checks if shcedule is enabled
		SCHEDULEINFO si;
		GetScheduleInfo(cf, si);
		return si.bEnable ? BKMENU_CMDUI_CHECKED : 0;
	}
	// Disables if current folder is mailbox root
	return BKMENU_CMDUI_DISABLED;
}

bool IsLockout(const SCHEDULEINFO& si, LPCSTR pszIni)
{
	char szLast[64];
	unsigned __int64 last;
	GetPrivateProfileString(INI_SECTION, "LastScheduled", "", szLast, countof(szLast), pszIni);
	last = hex64(szLast);
	if(si.bLockout && last != 0)
	{
		static unsigned __int64 units[] = { FT_HOUR, FT_DAY, FT_DAY * 7 };
		unsigned __int64 ft;
		GetSystemTimeAsFileTime((LPFILETIME)&ft);
		if(last + units[si.nLockoutUnit] * si.nLockoutNum > ft)
			return true;
	}
	return false;
}

void WriteCurTime(LPCSTR pszIni)
{
	unsigned __int64 ft;
	GetSystemTimeAsFileTime((LPFILETIME)&ft);
	char hex[32];
	hex64(hex, ft);
	WritePrivateProfileString(INI_SECTION, "LastScheduled", hex, pszIni);
}

// Schedule
bool DoSchedule(String folder, int nTrigger)
{
	CBeckyWindow wnd;
	SCHEDULEINFO si;
	String fini = bka.GetDataFolder() + folder + "Folder.ini";
	String restore;
	
	GetScheduleInfo(folder, si);
	if(!si.bEnable || si.nTrigger != nTrigger || IsLockout(si, fini))
		return false;
	
	bool bDo = true;
	if(si.nTrigger == IDC_ONSELECT)
	{
		// Count number of mails
		CFolderIndex idx(folder);
		if(idx.Failed())
			bDo = false;
		else
		{
			int n = 0;
			while(idx.NextLine())
				++n;
			bDo = n > si.nFMails;
		}
		// Confirm
		if(bDo)
			bDo = MyMessageBox(wnd.hwndMain, IDS_CONFIRM_SCHEDULE, MB_YESNO | MB_ICONQUESTION) == IDYES;
		if(bDo)
			restore = bka.GetCurrentMail();
	}
	if(bDo)
	{
		if(si.nTrigger == IDC_ONEND)
		{
			char msg[64];
			LoadString(g_hInstance, IDS_PROCESS_SCHEDULE, msg, countof(msg));
			SendMessage(wnd.hwndStatus, WM_SETREDRAW, TRUE, 0);
			bka.SetMessageText(NULL, String(String::concat(msg, bka.GetFolderDisplayName(folder))));
			SendMessage(wnd.hwndStatus, WM_SETREDRAW, FALSE, 0);
		}
		RMINFO ri;
		ri.name = folder;
		ri.nFilter = si.nFilter;
		if(ri.nFilter == IDC_FDAYS)
			ri.u.nDays = si.nDays;
		else
			ri.u.nMails = si.nMails;
		ri.bThreads = si.bThreads ? true : false;
		ri.nBehavior = si.nBehavior;
		ri.target = si.target;
		ri.bRecursive = si.bRecursive ? true : false;
		ri.pfi = &si.filter;
		
		RemoveMails(ri, nTrigger == IDC_ONSELECT);
		
		if(si.bLockout)
			WriteCurTime(fini);
	}
	if(restore != String::Nil)
	{
		bka.SetCurrentMail(restore);
		// Force activate ListView
		// ** MFC calls SetActiveView in OnMouseActivate handler
		SendMessage(wnd.hwndList, WM_MOUSEACTIVATE, (WPARAM)wnd.hwndMain,
			MAKELPARAM(HTBORDER, WM_MBUTTONDOWN));
	}
	return bDo;
}
