// setting.cpp - setting dialog
//
// Old Mail Arrangement Plug-in for Becky! 2
// Copyright (C) 2000 by Tietew
//
#include "pch.h"
#include "bkrmold.h"
#include "utys.h"

// Hand cursor
static HCURSOR g_hHandCursor;

// Window procedure for Anchor Text
LRESULT CALLBACK AnchorProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch(msg)
	{
		case WM_SETCURSOR:
			// The cursor is hand
			SetCursor(g_hHandCursor);
			return 0;
		
		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC dc = BeginPaint(hwnd, &ps);
			// Create underlined font
			LOGFONT lf;
			HFONT hFont = (HFONT)SendMessage(GetParent(hwnd), WM_GETFONT, 0, 0);
			GetObject(hFont, sizeof lf, &lf);
			lf.lfUnderline = TRUE;
			hFont = CreateFontIndirect(&lf);
			// Get text to draw
			int len = GetWindowTextLength(hwnd);
			char *str = (char*)_alloca(len + 1);
			len = GetWindowText(hwnd, str, len + 1);
			// Prepare DC
			COLORREF cr = SetTextColor(dc, RGB(0, 0, 255));
			HGDIOBJ hOldFont = SelectObject(dc, hFont);
			int bk = SetBkMode(dc, TRANSPARENT);
			// Draw
			RECT rc;
			GetClientRect(hwnd, &rc);
			DrawText(dc, str, len, &rc, DT_SINGLELINE | DT_NOPREFIX | DT_LEFT);
			// Unprepare DC
			SelectObject(dc, hOldFont);
			SetBkMode(dc, bk);
			SetTextColor(dc, cr);
			DeleteObject(hFont);
			// Finish
			EndPaint(hwnd, &ps);
			return 0;
		}
		case WM_LBUTTONDOWN:
			// Capture mouse
			SetCapture(hwnd);
			return 0;
		
		case WM_LBUTTONUP:
			if(GetCapture() == hwnd)
			{
				// Release capture and send notify if cursor is on the window
				RECT rect;
				ReleaseCapture();
				GetClientRect(hwnd, &rect);
				if((DWORD)LOWORD(lp) < (DWORD)rect.right && (DWORD)HIWORD(lp) < (DWORD)rect.bottom)
					SendMessage(GetParent(hwnd), WM_COMMAND, GetDlgCtrlID(hwnd), 0);
			}
			return 0;
	}
	return DefWindowProc(hwnd, msg, wp, lp);
}

BOOL CALLBACK AboutProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			// Load hwnd cursor
			if(!g_hHandCursor)
			{
				g_hHandCursor = LoadCursor(NULL, IDC_HAND);
				if(!g_hHandCursor)
					g_hHandCursor = LoadCursor(g_hInstance, MAKEINTRESOURCE(IDC_MYHAND));
			}
			char szVersion[64];
			GetVersionString(szVersion);
			SetDlgItemText(hwnd, IDC_VERSION, szVersion);
			// Subclass anchor
			SetWindowLong(GetDlgItem(hwnd, IDC_WEB), GWL_WNDPROC, (LONG)AnchorProc);
			SetWindowLong(GetDlgItem(hwnd, IDC_EMAIL), GWL_WNDPROC, (LONG)AnchorProc);
			break;
		}
		case WM_COMMAND:
			if(wp == IDOK || wp == IDCANCEL)
			{
				EndDialog(hwnd, wp);
				return TRUE;
			}
			else if(wp == IDC_WEB || IDC_EMAIL)
			{
				char url[MAX_PATH];
				GetDlgItemText(hwnd, wp, url, MAX_PATH);
				if(wp == IDC_WEB)
					ShellExecute(hwnd, NULL, url, NULL, NULL, SW_SHOW);
				else
					bka.ComposeMail(String(String::concat("mailto:", url)));
				return TRUE;
			}
			break;
	}
	return FALSE;
}

BOOL CALLBACK SettingProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			// Initialize controls
			SendDlgItemMessage(hwnd, IDC_DAYSSPIN, UDM_SETRANGE32, 0, INT_MAX);
			SendDlgItemMessage(hwnd, IDC_MAILSSPIN, UDM_SETRANGE32, 0, INT_MAX);
			SendDlgItemMessage(hwnd, IDC_LOCKOUTSPIN, UDM_SETRANGE32, 0, INT_MAX);
			SetDlgItemInt(hwnd, IDC_DAYS, GetIniDays(), FALSE);
			SetDlgItemInt(hwnd, IDC_MAILS, GetIniMails(), FALSE);
			CheckDlgButton2(hwnd, IDC_THREAD, GetIniThreads());
			CheckDlgButton2(hwnd, GetIniFilter() ? IDC_FLAST : IDC_FDAYS, TRUE);
			CheckDlgButton2(hwnd, IDFromBehavior(GetIniBehavior()), TRUE);
			CheckDlgButton2(hwnd, IDC_RECURSIVE, GetIniRecursive());
			
			SendDlgItemMessage(hwnd, IDC_HOT_ARRANGE, HKM_SETRULES,
				HKCOMB_A | HKCOMB_CA | HKCOMB_NONE | HKCOMB_SA | HKCOMB_SCA,
				MAKELPARAM(HOTKEYF_SHIFT | HOTKEYF_CONTROL, 0));
			SendDlgItemMessage(hwnd, IDC_HOT_ARRANGE, HKM_SETHOTKEY, GetIniHotArrange(), 0);
			SendDlgItemMessage(hwnd, IDC_HOT_SCHEDULE, HKM_SETRULES,
				HKCOMB_A | HKCOMB_CA | HKCOMB_NONE | HKCOMB_SA | HKCOMB_SCA,
				MAKELPARAM(HOTKEYF_SHIFT | HOTKEYF_CONTROL, 0));
			SendDlgItemMessage(hwnd, IDC_HOT_SCHEDULE, HKM_SETHOTKEY, GetIniHotSchedule(), 0);
			
			CheckDlgButton2(hwnd, IDC_EXITSCHEDULE, !GetIniExitSchedule());
			char unit[16];
			for(UINT i = IDS_UNIT1; i <= IDS_UNIT3; ++i)
			{
				LoadString(g_hInstance, i, unit, countof(unit));
				SendDlgItemMessage(hwnd, IDC_LOCKOUTUNIT, CB_ADDSTRING, 0, (LPARAM)unit);
			}
			CheckDlgButton2(hwnd, IDC_LOCKOUT, GetIniLockout());
			SetDlgItemInt(hwnd, IDC_LOCKOUTNUM, GetIniLockoutNum(), FALSE);
			SendDlgItemMessage(hwnd, IDC_LOCKOUTUNIT, CB_SETCURSEL, GetIniLockoutUnit(), 0);
			
			PostMessage(hwnd, WM_COMMAND, IDC_EXITSCHEDULE, 0);
			return TRUE;
		}
		case WM_COMMAND:
			switch(LOWORD(wp))
			{
				case IDOK:
					// Apply settings
					WriteIniInt(INI_SECTION, INI_KEY_DAYS,
						GetDlgItemInt(hwnd, IDC_DAYS, NULL, FALSE), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_MAILS,
						GetDlgItemInt(hwnd, IDC_MAILS, NULL, FALSE), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_THREADS,
						IsDlgButtonChecked2(hwnd, IDC_THREAD), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_BEHAVIOR,
						IsDlgButtonChecked2(hwnd, IDC_REMOVE)
						? 2 : IsDlgButtonChecked2(hwnd, IDC_MOVE) ? 1 : 0, g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_FILTER,
						IsDlgButtonChecked2(hwnd, IDC_FLAST), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_RECURSIVE,
						IsDlgButtonChecked2(hwnd, IDC_RECURSIVE), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_HOT_ARRANGE,
						SendDlgItemMessage(hwnd, IDC_HOT_ARRANGE, HKM_GETHOTKEY, 0, 0), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_HOT_SCHEDULE,
						SendDlgItemMessage(hwnd, IDC_HOT_SCHEDULE, HKM_GETHOTKEY, 0, 0), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_EXITSCHEDULE,
						!IsDlgButtonChecked2(hwnd, IDC_EXITSCHEDULE), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_LOCKOUT,
						IsDlgButtonChecked2(hwnd, IDC_LOCKOUT), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_LOCKOUTNUM,
						GetDlgItemInt(hwnd, IDC_LOCKOUTNUM, NULL, FALSE), g_szIni);
					WriteIniInt(INI_SECTION, INI_KEY_LOCKOUTUNIT,
						SendDlgItemMessage(hwnd, IDC_LOCKOUTUNIT, CB_GETCURSEL, 0, 0), g_szIni);
				// fall through
				case IDCANCEL:
					EndDialog(hwnd, wp);
					return TRUE;
				
				case IDC_EXITSCHEDULE:
				case IDC_LOCKOUT:
				{
					bool bEnable = !IsDlgButtonChecked2(hwnd, IDC_EXITSCHEDULE);
					bool eLockout = bEnable && IsDlgButtonChecked2(hwnd, IDC_LOCKOUT);
					EnableWindow(GetDlgItem(hwnd, IDC_LOCKOUT), bEnable);
					EnableWindow(GetDlgItem(hwnd, IDC_LOCKOUTNUM), eLockout);
					EnableWindow(GetDlgItem(hwnd, IDC_LOCKOUTUNIT), eLockout);
					return TRUE;
				}
				case IDC_ABOUT:
					DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_ABOUT), hwnd, (DLGPROC)AboutProc);
					return TRUE;
			}
			break;
	}
	return FALSE;
}

// Show dialog
void SettingDlg(HWND hwnd)
{
	DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_SETTING), hwnd, (DLGPROC)SettingProc);
}
