// utys.cpp - utilities
//
// Old Mail Arrangement Plug-in for Becky! 2
// Copyright (C) 2000 by Tietew
//
#include "pch.h"
#include "bkrmold.h"
#include "utys.h"

// Parse string value
String token(LPCSTR& str)
{
	LPCSTR first = str, last;
	while(*str && *str != 0x01)
		++str;
	last = str;
	if(*str == 0x01)
		++str;
	return String(first, last);
}

// Parse 32-bit hex value
DWORD hex32(LPCSTR& str)
{
	DWORD dw = 0;
	while(*str && (*str >= '0' && *str <= '9' || *str >= 'A' && *str <= 'F' || *str >= 'a' && *str <= 'f'))
	{
		dw <<= 4;
		dw |= *str >= 'a' ? *str - 'a' + 10 : (*str >= 'A' ? *str - 'A' + 10 : *str - '0');
		++str;
	}
//	token(str);
	return dw;
}

// Generate 32-bit hex value
String hex32(DWORD dw)
{
	String r;
	LPSTR str = r.Alloc(8);
	for(int i = 7; i >= 0; --i)
	{
		str[i] = "0123456789ABCDEF"[dw & 0x0F];
		dw >>= 4;
	}
	str[8] = '\0';
	return r.Fix(8);
}

//////////////////////////////////////////////////////////////
// CFolderIndex class
//

// constructor
CFolderIndex::CFolderIndex(String folder)
{
	m_hMap = NULL;
	m_pch = NULL;
	m_dwCurOffset = 0;
	// Open Folder.idx
	m_hFile = CreateFile(String(bka.GetDataFolder() + folder + "Folder.idx"),
		GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(m_hFile != INVALID_HANDLE_VALUE)
	{
		// Folder.idx must be smaller than 4GB.
		DWORD dwHigh;
		m_dwSize = GetFileSize(m_hFile, &dwHigh);
		if(!dwHigh)
		{
			// Create as memory mapped file
			m_hMap = CreateFileMapping(m_hFile, NULL, PAGE_READONLY, 0, 0, NULL);
			if(m_hMap)
			{
				m_pch = (LPCSTR)MapViewOfFile(m_hMap, FILE_MAP_READ, 0, 0, m_dwSize);
				if(m_pch)
				{
					// Folder.idx version check
					if(memcmp(m_pch, "2.00", 4) == 0)
					{
						m_dwCurOffset = 4;
						while(m_pch[m_dwCurOffset] != '\n')
							++m_dwCurOffset;
						++m_dwCurOffset;
						return;
					}
					m_dwCurOffset = -1;
					UnmapViewOfFile(m_pch);
				}
				CloseHandle(m_hMap);
			}
		}
		CloseHandle(m_hFile);
		m_hFile = INVALID_HANDLE_VALUE;
	}
}

// destructor
CFolderIndex::~CFolderIndex()
{
	if(!Failed())
	{
		if(m_pch)
			UnmapViewOfFile(m_pch);
		CloseHandle(m_hMap);
		CloseHandle(m_hFile);
	}
}

String CFolderIndex::GetLine()
{
	LPCSTR pch = (LPCSTR)memchr(m_pch + m_dwCurOffset, '\n', m_dwSize - m_dwCurOffset);
	if(!pch)
		return String::Nil;
	DWORD dwOffset = m_dwCurOffset;
	m_dwCurOffset += (pch - m_pch) - m_dwCurOffset + 1;
	return String(m_pch + dwOffset, pch - 2);
}

bool CFolderIndex::NextLine()
{
	LPCSTR pch = (LPCSTR)memchr(m_pch + m_dwCurOffset, '\n', m_dwSize - m_dwCurOffset);
	if(!pch)
		return false;
	m_dwCurOffset += (pch - m_pch) - m_dwCurOffset + 1;
	return true;
}

// Parse hexiadecimal value
static DWORD hex(LPCSTR& str)
{
	DWORD dw = 0;
	while(*str && *str != 0x01)
	{
		dw *= 0x10;
		dw += (*str >= 'a') ? (*str - 'a' + 10) : (*str >= 'A' ? (*str - 'A' + 10) : (*str - '0'));
		++str;
	}
	if(*str == 0x01)
		++str;
	return dw;
}

// Parse line
void CFolderIndex::Parse(LPCSTR line, INDEXINFO& info)
{
	info.dwOffset = hex(line);
	info.id = token(line);
	info.bmf = token(line);
	info.subject = token(line);
	info.from = token(line);
	info.to = token(line);
	info.messageid = token(line);
	info.reference = token(line);
	info.dwDate = hex(line);
	info.dwRecvDate = hex(line);
	info.dwRouteDate = hex(line);
	info.dwSize = hex(line);
	info.dwStatus = hex(line);
	info.color = hex(line);
	info.nPriority = hex(line);
	info.refid = token(line);
	info.lang = token(line);
}

///////////////////////////////////////////////////////
// CBeckyWindow class

CBeckyWindow::CBeckyWindow()
{
	// Get window handles and status-bar handle
	bka.GetWindowHandles(&hwndMain, &hwndTree, &hwndList, &hwndView);
	hwndStatus = FindWindowEx(hwndMain, NULL, STATUSCLASSNAME, NULL);
	hwndListHead = FindWindowEx(hwndList, NULL, WC_HEADER, NULL);
}

///////////////////////////////////////////////////////
// Functions

// Same as DrawFocusRect...
void PseudoDrawFocusRect(HDC hdc, LPRECT prc)
{
	static WORD pat[] = {0xAAAA,0x5555,0xAAAA,0x5555,0xAAAA,0x5555,0xAAAA,0x5555};
	HBITMAP hbm = CreateBitmap(8, 8, 1, 1, pat);
	HBRUSH hbr = CreatePatternBrush(hbm);
	COLORREF crOld = SetTextColor(hdc, 0);
	int r2Old = SetROP2(hdc, R2_XORPEN);
	FrameRect(hdc, prc, hbr);
	SetROP2(hdc, r2Old);
	SetTextColor(hdc, crOld);
	DeleteObject(hbr);
	DeleteObject(hbm);
}

int MyMessageBox(HWND hwnd, UINT uID, UINT uType)
{
	char msg[128], title[32];
	LoadString(g_hInstance, IDS_NAME, title, sizeof title);
	LoadString(g_hInstance, uID, msg, sizeof msg);
	return MessageBox(hwnd, msg, title, uType);
}

void GetVersionString(LPSTR pszVersion)
{
	DWORD dw, dwSize;
	char path[MAX_PATH];
	GetModuleFileName(g_hInstance, path, MAX_PATH);
	dwSize = GetFileVersionInfoSize(path, &dw);
	void *pvVerInfo = _alloca(dwSize);
	GetFileVersionInfo(path, 0, dwSize, pvVerInfo);
	
	// Make version string from version information
	UINT uLen;
	VS_FIXEDFILEINFO *pffi;
	VerQueryValue(pvVerInfo, "\\", (void**)&pffi, &uLen);
	wsprintf(pszVersion, "Version %d.%d%c",
		pffi->dwFileVersionMS >> 16,
		pffi->dwFileVersionMS & 0xFFFF,
		(pffi->dwFileVersionLS >> 16) ? (pffi->dwFileVersionLS >> 16) + '`' : '\0');
}
